@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+import archspec.cpu
 import argparse
 import os
 import traceback
@@ -49,7 +50,7 @@ def main():  # pragma: no cover
         "--base_image",
         type=str,
         default="ubuntu:20.04",
-        help="The base image. (centos:8 is supported too)",
+        help="The base image.",
     )
     general_g.add_argument(
         "--runtime_base_image",
@@ -57,6 +58,13 @@ def main():  # pragma: no cover
         default="",
         help="The runtime base image.",
     )
+    general_g.add_argument(
+        "--cpu-target",
+        type=str,
+        default=None,
+        choices=[a for a in sorted(archspec.cpu.TARGETS)],
+        help="The CPU microarchitecture to optimize for.",
+    )
     build_g = parser.add_argument_group("Image build options")
     build_g.add_argument(
         "--build",
@@ -179,6 +187,7 @@ def main():  # pragma: no cover
     if not os.path.exists(images_out_dir):
         os.makedirs(images_out_dir)
 
+    hpccm.config.set_cpu_target(args.cpu_target)
     Stage0 = hpccm.Stage()
     Stage0 += raw(docker="# syntax=docker/dockerfile:experimental")
 
@@ -189,7 +198,9 @@ def main():  # pragma: no cover
     Stage0 += comment(
         f"Generated with ogs-container-maker {__version__}", reformat=False
     )
-    Stage0 += packages(ospackages=["wget", "tar", "curl", "make", "unzip"])
+    Stage0 += packages(
+        ospackages=["wget", "tar", "curl", "ca-certificates", "make", "unzip"]
+    )
 
     # Prepare runtime stage
     Stage1 = hpccm.Stage()
@@ -268,14 +279,14 @@ def main():  # pragma: no cover
         os.makedirs(out_dir)  # For .scif files
 
     # General args
-    if args.pip:
-        Stage0 += pip(packages=args.pip, pip="pip3")
-        Stage1 += pip(packages=args.pip, pip="pip3")
-
     if args.packages:
         Stage0 += packages(ospackages=args.packages)
         Stage1 += packages(ospackages=args.packages)
 
+    if args.pip:
+        Stage0 += pip(packages=args.pip, pip="pip3")
+        Stage1 += pip(packages=args.pip, pip="pip3")
+
     # Create definition
     hpccm.config.set_container_format(args.format)
 
@@ -286,7 +297,11 @@ def main():  # pragma: no cover
         if hasattr(args, "mfront") and not args.mfront:
             runtime_exclude.append("boost")
         Stage1 += Stage0.runtime(exclude=runtime_exclude)
-        if args.compiler == "gcc" and args.compiler_version != None:
+        if (
+            hasattr(args, "compiler")
+            and args.compiler == "gcc"
+            and args.compiler_version != None
+        ):
             Stage1 += packages(apt=["libstdc++6"])
         if args.runtime_base_image == "jupyter/base-notebook":
             Stage1 += raw(docker="USER ${NB_USER}")
