@@ -117,6 +117,13 @@ parse_g.add_argument(
     action="store_true",
     help="Use MKL. By setting this option, you agree to the [Intel End User License Agreement](https://software.intel.com/en-us/articles/end-user-license-agreement).",
 )
+if toolchain.CC == "mpicc":
+    parse_g.add_argument(
+        "--petsc_configure_args",
+        type=str,
+        default="--with-fc=0 --download-f2cblaslapack=1",
+        help="PETSc configuration arguments; has to be quoted.",
+    )
 parse_g.add_argument("--version_file", type=str, help="OGS versions.json file")
 
 # Parse local args
@@ -127,6 +134,11 @@ git_version = ""
 name_start = ""
 repo = None
 versions = None
+versions_master = json.loads(
+    requests.get(
+        f"https://gitlab.opengeosys.org/ogs/ogs/-/raw/master/web/data/versions.json?inline=false"
+    ).text
+)
 
 if local_args.ogs not in ["off", "clean"]:  # != "off" and local_args.ogs != "clean":
     if os.path.isdir(local_args.ogs):
@@ -260,7 +272,10 @@ if local_args.gui:
         yum=["mesa-libOSMesa", "mesa-libGL", "mesa-libGLU", "libXt"],
     )
 if local_args.ogs != "clean":
-    Stage0 += cmake(eula=True, version="3.21.0")
+    cmake_version = versions_master["tested_version"]["cmake"]
+    if "cmake" in versions["tested_version"]:
+        cmake_version = versions["tested_version"]["cmake"]
+    Stage0 += cmake(eula=True, version=cmake_version)
     if local_args.pm == "conan":
         conan_user_home = "/opt/conan"
         if local_args.dev:
@@ -273,14 +288,15 @@ if local_args.ogs != "clean":
         boost_bootsrap_opts = []
         if local_args.mfront:
             # for mfront python bindings
-            boost_bootsrap_opts = ['--with-python=python3']
+            boost_bootsrap_opts = ["--with-python=python3"]
         if toolchain.CC == "clang":
             boost_bootsrap_opts.append("--with-toolset=clang")
         Stage0 += boost(
-            baseurl=f"https://boostorg.jfrog.io/artifactory/main/release/{versions['minimum_version']['boost']}/source",
-            version=versions["minimum_version"]["boost"],
-            bootstrap_opts=boost_bootsrap_opts,
             b2_opts=["headers"],
+            baseurl=f"https://boostorg.jfrog.io/artifactory/main/release/{versions['minimum_version']['boost']}/source",
+            bootstrap_opts=boost_bootsrap_opts,
+            ldconfig=True,
+            version=versions["minimum_version"]["boost"],
         )
         Stage0 += environment(variables={"BOOST_ROOT": "/usr/local/boost"})
         Stage0 += packages(
@@ -321,7 +337,7 @@ if local_args.ogs != "clean":
                     "libx11-xcb1",
                     "libxkbcommon-x11-0",
                 ],
-                # TODO: Add runtime packages for centos
+                # TODO: Add runtime packages for yum
                 yum=[
                     "libgeotiff",
                     "shapelib",
@@ -332,7 +348,7 @@ if local_args.ogs != "clean":
             qt_install_dir = "/opt/qt"
             qt_version = versions["minimum_version"]["qt"]
             qt_dir = f"{qt_install_dir}/{qt_version}/gcc_64"
-            Stage0 += pip(pip="pip3", packages=["aqtinstall"])
+            Stage0 += pip(pip="pip3", packages=["aqtinstall==1.2.5"])
             Stage0 += shell(
                 commands=[
                     f"aqt install --outputdir {qt_install_dir} {qt_version} linux desktop -m xmlpatterns,x11extras"
@@ -399,17 +415,21 @@ if local_args.ogs != "clean":
         if toolchain.CC == "mpicc":
             Stage0 += packages(yum=["diffutils"])
             petsc_version = versions["minimum_version"]["petsc"]
+            petsc_args = local_args.petsc_configure_args.strip().split(" ")
+            petsc_configure_opts = [
+                f"CC={toolchain.CC}",
+                f"CXX={toolchain.CXX}",
+                f"FC={toolchain.FC}",
+                f"F77={toolchain.F77}",
+                f"F90={toolchain.F90}",
+                "--CFLAGS='-O3'",
+                "--CXXFLAGS='-O3'",
+                "--FFLAGS='-O3'",
+                "--with-debugging=no",
+            ]
+            petsc_configure_opts.extend(petsc_args)
             Stage0 += generic_autotools(
-                configure_opts=[
-                    f"CC={toolchain.CC}",
-                    f"CXX={toolchain.CXX}",
-                    "--CFLAGS='-O3'",
-                    "--CXXFLAGS='-O3'",
-                    "--FFLAGS='-O3'",
-                    "--with-debugging=no",
-                    "--with-fc=0",
-                    "--download-f2cblaslapack=1",
-                ],
+                configure_opts=petsc_configure_opts,
                 devel_environment={"PETSC_DIR": "/usr/local/petsc"},
                 directory=f"petsc-{petsc_version}",
                 ldconfig=True,
@@ -427,15 +447,20 @@ if local_args.ogs != "clean":
             },
             directory=f"eigen-{eigen_version}",
             prefix="/usr/local/eigen",
+            toolchain=toolchain,
             url=f"https://gitlab.com/libeigen/eigen/-/archive/{eigen_version}/eigen-{eigen_version}.tar.gz",
         )
         hdf5_cofigure_opts = ["--enable-cxx"]
         if toolchain.CC == "mpicc":
             hdf5_cofigure_opts = ["--enable-parallel", "--enable-shared"]
+        hdf5_version = versions_master["minimum_version"]["hdf5"]
+        if "hdf5" in versions["minimum_version"]:
+            hdf5_version = versions["minimum_version"]["hdf5"]
         Stage0 += hdf5(
             configure_opts=hdf5_cofigure_opts,
+            ldconfig=True,
             toolchain=toolchain,
-            version=versions["minimum_version"]["hdf5"],
+            version=hdf5_version,
         )
 if local_args.cvode:
     # TODO version
@@ -448,6 +473,7 @@ if local_args.cvode:
         devel_environment={"CVODE_ROOT": "/usr/local/cvode"},
         directory="cvode-2.8.2",
         prefix="/usr/local/cvode",
+        toolchain=toolchain,
         url="https://github.com/ufz/cvode/archive/2.8.2.tar.gz",
     )
 
@@ -457,6 +483,7 @@ if local_args.cppcheck:
         directory="cppcheck-809a769c690d8ab6fef293e41a29c8490512866e",
         prefix="/usr/local/cppcheck",
         runtime_environment={"PATH": "/usr/local/cppcheck/bin:$PATH"},
+        toolchain=toolchain,
         url="https://github.com/danmar/cppcheck/archive/809a769c690d8ab6fef293e41a29c8490512866e.tar.gz",
     )
 
@@ -471,21 +498,24 @@ if local_args.dev:
     )
 
 if local_args.mfront:
-    tfel_version = versions["minimum_version"]["tfel-rliv"]
+    tfel_version = versions_master["minimum_version"]["tfel-rliv"]
+    if "tfel-rliv" in versions["minimum_version"]:
+        tfel_version = versions["minimum_version"]["tfel-rliv"]
     Stage0 += generic_cmake(
-        cmake_opts=['-Denable-python-bindings=ON'],
+        cmake_opts=["-Denable-python-bindings=ON"],
         directory=f"tfel-rliv-{tfel_version}",
         ldconfig=True,
         url=f"https://github.com/thelfer/tfel/archive/refs/heads/rliv-{tfel_version}.zip",
         prefix="/usr/local/tfel",
         runtime_environment={"PATH": "/usr/local/tfel/bin:$PATH"},
         devel_environment={"PATH": "/usr/local/tfel/bin:$PATH"},
+        toolchain=toolchain,
     )
     tfel_env = environment(
         variables={
             "TFELHOME": "/usr/local/tfel",
             # TODO: Don't hard-code python version
-            "PYTHONPATH": "/usr/local/tfel/lib/python3.8/site-packages:$PYTHONPATH"
+            "PYTHONPATH": "/usr/local/tfel/lib/python3.8/site-packages:$PYTHONPATH",
         }
     )
     Stage0 += tfel_env
@@ -494,6 +524,8 @@ if local_args.mfront:
 
 if local_args.mkl:
     Stage0 += mkl(eula=True, mklvars=False)
+    # For 32-bit mkl_link_tool:
+    Stage0 += packages(apt=["lib32stdc++6"], yum=["libstdc++.i686"])
     cmake_args.append("-DOGS_USE_MKL=ON")
 
 if local_args.ccache:
@@ -527,3 +559,8 @@ if local_args.ogs != "off" and local_args.ogs != "clean":
         remove_build=True,
         remove_source=True,
     )
+
+# Required for vtk from Python (for notebooks, VTUInterface)
+# https://github.com/Kaggle/docker-python/pull/358
+Stage0 += packages(apt=["libgl1"], yum=["mesa-libGL"])
+Stage1 += packages(apt=["libgl1"], yum=["mesa-libGL"])
